!    These codes uses an implicit CrankCNicolson finite difference scheme to solve the parabolic equation 
!    with a non-local boundary condition. The theoretical framework of this problem was addressed by 
!    Yevick and Thomson (Nonlocal boundary conditions for finite difference parabolic equation solvers, JASA, 1999).
!    Both the tangent linear codes and the adjoint codes of NLBC PE are also provided. So it is very convenient to use
!    or modify these codes for inverse problems, such as ocean acoustic tomography (Zhao and Wang, Ocean acoustic 
!    tomography from different receiver geometries using the adjoint method, JASA, 2015) or geoacoustic inversions.
!
!    These codes were developed by Xiaofeng ZHAO and the copyright belongs to PLA University of Sci. & Tech. 
!    The lastest version was updated at Nov. 8, 2015. Any publications using these codes are encouraged to 
!    cite Ocean acoustic tomography from different receiver geometries using the adjoint method.
!
!    These codes are just provided for research purpose. Anyone cannot use these codes for business purpose.

    
    program main
    
    use gridmod
    
    implicit integer(kind=4) (i-n) 
    implicit real(kind=8)    (a-h, o-z)    
      
    ! control variables
    real(kind=8)      cwater(n), cwaterd(n), cwaterb(n)
    character*60      filenm
    
    ! common variables
    common / source / frq, zs, zr, c0, xko
    common / layer1 / rho1, alpha1
    common / layer2 / zb, cb, rho2, alpha2
    
    ! used to test tangent linear and adjoint code
    real(kind=8)    u1(n), u2(n), u3(n), u4(n)   
    
!   environement parameters
    !write(*,'(a\)') 'Read the input file name for initialization: '
    !read(*,*) filenm
    filenm = 'input.inp'
    call initial( 0, filenm, cwater )

!   marching forward
    uobs = (0.d0,0.d0)
    pobs = 0.d0
    uobs(:,0) = uinit 
    call pestep(cwater)
    open(11, file='loss.out')
    do istep=1, nstep    
        if ( mod(istep,10) .eq. 0 ) then
            write(11,*) 'Range = ', istep*dr
            do j=1, n
                write(11,'(f10.1,2x,f20.15)') j*dz, pobs(j,istep)
            end do
        end if    
    end do    
    close(11)    
    
    !stop
    
!   test tangent linear code
    call random_seed()
    do i=1, n
        !call random_number(ttt)
        !cwaterd(i) = 2.d0 * cwater(i) * (2.0*ttt - 1.0) 
        cwaterd(i) = 20. * dsin(i*2*pi/n)  
    end do
    
    write(*,*) 'test tangent linear mode:'
    open(12,file='tangent.dat')
    alpc = 3.d-1
    do k=0, 20
        
        uobs = (0.d0,0.d0)
        pobs = 0.d0
        uobs(:,0) = uinit 
        call pestep( cwater+cwaterd*(alpc**k) )
        u2(10:210:2) = pobs(10:210:2,nstep)
              
        uobs = (0.d0,0.d0)
        pobs = 0.d0
        uobs(:,0) = uinit 
        call pestep( cwater )
        u1(10:210:2) = pobs(10:210:2,nstep)

        call pestep_d(cwater, cwaterd*(alpc**k))
        u3(10:210:2) = pobsd(10:210:2,nstep)
        
        sum1 = 0.d0
        sum2 = 0.d0
        sum3 = 0.d0
        do i = 10, 210, 2
            sum1 = sum1 + dabs( u1(i) - u2(i) )
            sum2 = sum2 + dabs( u3(i) )
        end do
        
        write(*,'(i,2x,e14.7,2x,e14.7,2x,e14.7)') k, sum1, sum2, sum1/sum2
        write(12,'(2x, e14.7,2x,e14.7,2x,e14.7,2x,e14.7)') alpc**k, sum1, sum2, sum1/sum2
        
    end do
    close(12)
    
!   test adjoint code

    write(*,*) 'test adjoint code:'
    open(13,file='adjoint.dat')
    
    uobs = (0.d0,0.d0)
    pobs = 0.d0    
    uobs(:,0) = uinit 
    call pestep(cwater) 
        
    do k=0, 20
       
        call pestep_d( cwater, cwaterd*(alpc**k) )
        u1(10:210:2) = pobsd(10:210:2,nstep)
        sum1 = 0.d0
        do i = 10, 210, 2
            sum1 = sum1 + u1(i) * u1(i) 
        end do        
 
        pobsb = 0.d0
        cwaterb = 0.d0
        pobsb(10:210:2,nstep) = u1(10:210:2)
        call  PESTEP_B(cwater, cwaterb)
        sum2 = 0.d0
        do i = 1, n
            sum2 = sum2 + cwaterd(i)*(alpc**k) * cwaterb(i)
        end do
         
        write(*,'(i,2x,e14.7,2x,e14.7,2x,e14.7)') k, sum1, sum2, sum1/sum2
        write(13,'(e14.7,2x,e14.7,2x,e14.7,2x,e14.7)') alpc**k, sum1, sum2, sum1/sum2
        
    end do
    close(13)
    
!   test gradient
    write(*,*) 'test gradient:'
    open(14,file='gradient.dat')
        
    do k=0, 20
        
        uobs = (0.d0,0.d0)
        pobs = 0.d0
        uobs(:,0) = uinit 
        call pestep( cwater+cwaterd*(alpc**k) )
        u1(10:210:2) = pobs(10:210:2,nstep)   
 
        sum1 = 0.d0
        do i = 10, 210, 2
            sum1 = sum1 + 0.5*dabs( u1(i) )**2
        end do
        
        uobs = (0.d0,0.d0)
        pobs = 0.d0
        uobs(:,0) = uinit 
        call pestep( cwater )
        u2(10:210:2) = pobs(10:210:2,nstep) 
        
        sum2 = 0.d0
        do i = 10, 210, 2
            sum2 = sum2 + 0.5*dabs( u2(i) )**2
        end do

        pobsb = 0.d0
        cwaterb = 0.d0        
        pobsb(10:210:2,nstep) = u2(10:210:2)
        call  PESTEP_B(cwater, cwaterb)
        
        sum3 = 0.d0
        do i = 1, n
            sum3 = sum3 + cwaterd(i)*(alpc**k) * cwaterb(i) 
        end do
         
        write(*,'(i,2x,e14.7,2x,e14.7,2x,e14.7,2x,e14.7)') k, sum1-sum2, sum3, (sum1-sum2)/sum3
        write(14,'(e14.7,2x,e14.7,2x,e14.7,2x,e14.7)') alpc**k, sum1-sum2, sum3, (sum1-sum2)/sum3
        
    end do
    close(14)
    
    stop
    
    end program main 
